@@ -13,12 +13,16 @@
 # OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 
 from PySide.QtCore import *
+#from PySide.QtGui import QDesktopServices
 from PySide.QtGui import *
 import sys
-from foursquare import *
 import foursquare
+from foursquare import Cache
+from gui import WaitingDialog
 from locationProviders import LocationProvider
 from custom_widgets import SignalEmittingValueButton
+from threads import TipMarkTodoBackgroundThread, TipMarkDoneBackgroundThread
+import time
 
 try:
 	from PySide.QtMaemo5 import *
@@ -26,57 +30,9 @@ try:
 except ImportError:
 	maemo = False
 
-class CheckinConfirmation(QMessageBox):
-	def __init__(self, parent, venue):
-		super(CheckinConfirmation, self).__init__(parent)
+from checkins import CheckinConfirmation
+from checkins import CheckinDetails
 		
-		self.setWindowTitle("Confirmation")
-		text = "Do you want to check-in at <b>" + venue[u'name'] + "</b>"
-		if u'address' in venue[u'location']:
-			text += " located at " + venue[u'location'][u'address']	
-		text += "?"
-		self.setText(text)
-
-		self.addButton("Yes", QMessageBox.YesRole)
-		self.addButton("No", QMessageBox.NoRole)
-		self.setIcon(QMessageBox.Question)
-		
-class CheckinDetails(QDialog):
-	def __init__(self, parent, checking_details):
-		super(CheckinDetails, self).__init__(parent)
-		self.checking_details = checking_details
-		self.setWindowTitle("Check-in successful")
-
-		message = ""
-		score = ""
-		mayorship = ""
-		badge = ""
-
-		for item in checking_details[u'notifications']:
-			if item[u'type'] == "message":
-				message += item[u'item'][u'message'] + "<p>"
-			elif item[u'type'] == "score":
-				score += "Total points: %d" % item[u'item'][u'total']
-				for scoreItem in item[u'item'][u'scores']:
-					score += "<br>+%(points)d   %(message)s" % \
-					{'points': scoreItem[u'points'], 'message' : scoreItem[u'message']}
-				score += "<p>"
-			elif item[u'type'] == "mayorship":
-				mayorship = item[u'item'][u'message']
-			elif item[u'type'] == "badge":
-				badge = "<p>" + item[u'item'][u'message']
-
-		text = message + score + mayorship + badge
-
-		vbox = QVBoxLayout()
-		vbox.addWidget(QLabel(text))
-	
-		ok_button = QPushButton("Ok")
-		self.connect(ok_button, SIGNAL("clicked()"), self.close)
- 
-		vbox.addWidget(ok_button)
-		self.setLayout(vbox)
-
 class VenueListModel(QAbstractListModel):
 	def __init__(self, venues):
 		super(VenueListModel, self).__init__()
@@ -91,7 +47,11 @@ class VenueListModel(QAbstractListModel):
 		venue = self.venues[index.row()][u'venue']
 		if role == Qt.DisplayRole:
 			if u'address' in venue[u'location']:
-				return venue[u'name'] + "\n  " + venue[u'location'][u'address']
+				result = venue[u'name'] + "\n  " + venue[u'location'][u'address']
+				if len(result) < 72:
+					return result
+				else:
+					return result[0:67] + "[...]"
 			else:
 				return venue[u'name']
 		elif role == Qt.DecorationRole:
@@ -120,7 +80,12 @@ class VenueList(QListView):
 		self.clicked.connect(self.venue_selected)
 
 	def venue_selected(self, index):
-		d = VenueDetailsWindow(self, self.proxy.data(index,VenueListModel.VenueRole))
+		venue = self.proxy.data(index,VenueListModel.VenueRole)
+		cachedVenue = foursquare.venues_venue(venue[u'id'], foursquare.CacheOnly)
+		if not cachedVenue:
+			d = VenueDetailsWindow(self, venue, False)
+		else:
+			d = VenueDetailsWindow(self, cachedVenue, True)
 		d.show()
 
 	def filter(self,text):
@@ -131,7 +96,7 @@ class VenueList(QListView):
 		self.model.setVenues(venues)
 
 class VenueListWindow(QMainWindow):
-	def __init__(self, parent, title, venues):
+	def __init__(self, title, venues, parent):
 		super(VenueListWindow, self).__init__(parent)
 		if maemo:
 			self.setAttribute(Qt.WA_Maemo5StackedWindow)
@@ -154,9 +119,16 @@ class VenueListWindow(QMainWindow):
 
 		updateVenues = Signal()
 		self.connect(self, SIGNAL("updateVenues()"), self._updateVenues)
+		self.shown = False
+
+	def show(self):
+		super(VenueListWindow, self).show()
+		self.shown = True
 
 	def _updateVenues(self):
 		self.setVenues(self.parent().venues())
+		if not self.shown:
+			self.show()
 
 	def filter(self, text):
 		self.list.filter(text)
@@ -164,13 +136,78 @@ class VenueListWindow(QMainWindow):
 	def setVenues(self, venues):
 		self.list.setVenues(venues)
 
+class Tip(QWidget):
+	def __init__(self, tip, parent = None):
+		super(Tip, self).__init__(parent)
+
+		gridLayout = QGridLayout() 
+		self.setLayout(gridLayout) 
+		self.tip = tip
+
+		tipLabel = QLabel(tip[u'text'], self)
+		tipLabel.setWordWrap(True)
+		gridLayout.addWidget(tipLabel, 0, 0, 2, 1)
+
+		self.done_checkbox = QCheckBox("Done! (" + str(tip[u'done'][u'count']) + ")")
+		self.done_checkbox.setChecked(self.isTipDone())
+		self.done_checkbox.stateChanged.connect(self.markDone)
+
+		self.todo_checkbox = QCheckBox("To-do (" + str(tip[u'todo'][u'count']) + ")")
+		self.todo_checkbox.setChecked(self.isTipTodo())
+		self.todo_checkbox.stateChanged.connect(self.markTodo)
+
+		gridLayout.addWidget(self.done_checkbox, 0, 1, 1, 1)
+		gridLayout.addWidget(self.todo_checkbox, 1, 1, 1, 1)
+		gridLayout.setColumnStretch(0, 1)
+
+	def isTipInGroupType(self, groupType):
+		if u'listed' in self.tip:
+			if u'groups' in self.tip[u'listed']:
+				for group in self.tip[u'listed'][u'groups']:
+					if group[u'type'] == groupType:
+						return True
+		return False
+
+	def isTipDone(self):
+		return self.isTipInGroupType("dones")
+		
+	def isTipTodo(self):
+		return self.isTipInGroupType("todos")
+
+	def markTodo(self, state):
+		TipMarkTodoBackgroundThread(self.tip[u'id'], self.todo_checkbox.isChecked(), self).start()
+
+	def markDone(self, state):
+		TipMarkDoneBackgroundThread(self.tip[u'id'], self.done_checkbox.isChecked(), self).start()
+
+class NewTipWidget(QWidget):
+	def __init__(self, venueId, parent = None):
+		super(NewTipWidget, self).__init__(parent)
+
+		gridLayout = QGridLayout() 
+		self.setLayout(gridLayout) 
+		self.venueId = venueId
+
+		self.tip_text = QLineEdit()
+		self.tip_text.setPlaceholderText("Write something and then...")
+		gridLayout.addWidget(self.tip_text, 0, 0)
+
+		self.tip_button = QPushButton("TODO: Leave tip!")
+		self.tip_button.clicked.connect(self.addTip)
+		gridLayout.addWidget(self.tip_button, 0, 1)
+
+	def addTip(self):
+		# TODO: add tip self.tip_text.text()
+		pass
+
 class VenueDetailsWindow(QMainWindow):
-	def __init__(self, parent, venue, full = False):
+	def __init__(self, parent, venue, fullDetails):
 		super(VenueDetailsWindow, self).__init__(parent)
-		if maemo:
-			self.setAttribute(Qt.WA_Maemo5StackedWindow)
+		self.setAttribute(Qt.WA_Maemo5StackedWindow)
 		self.venue = venue
 
+		self.fullDetails = fullDetails
+
 		self.setWindowTitle(venue[u'name'])
 
 		self.centralWidget = QWidget() 
@@ -180,10 +217,11 @@ class VenueDetailsWindow(QMainWindow):
 		layout.setSpacing(0)         
 		self.centralWidget.setLayout(layout) 
 
-		self.container = QWidget() 
+		self.container = QWidget()
 
 		self.scrollArea = QScrollArea() 
-		self.scrollArea.setWidget(self.container)           
+		self.scrollArea.setWidget(self.container)
+		self.scrollArea.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
 
 		layout.addWidget(self.scrollArea)   
 
@@ -192,9 +230,13 @@ class VenueDetailsWindow(QMainWindow):
 		gridLayout = QGridLayout() 
 		self.container.setLayout(gridLayout) 
 
-		address = ""
-		address2 = ""
+		# name
+		name = "<b>" + venue[u'name'] + "</b>"
+		if len(venue[u'categories']) > 0:
+			name += " (" + venue[u'categories'][0][u'name'] + ")"
 
+		# address
+		address = ""
 		if u'address' in venue[u'location']:
 			address = venue[u'location'][u'address']
 
@@ -203,6 +245,8 @@ class VenueDetailsWindow(QMainWindow):
 				address += ", "
 			address += venue[u'location'][u'crossStreet']
 
+		# address2
+		address2 = ""
 		if u'postalCode' in venue[u'location']:
 			address2 = venue[u'location'][u'postalCode']
 
@@ -211,45 +255,163 @@ class VenueDetailsWindow(QMainWindow):
 				address2 += ", "
 			address2 += venue[u'location'][u'city']
 
+		# times
+		if u'beenHere' in venue:
+			if not fullDetails:
+				count = venue[u'beenHere']
+			else:
+				count = venue[u'beenHere']['count']
+			times = "<b>You've been here "
+			if count == 1:
+				times += "once"
+			else:
+				times += str(count) +" times"
+			times += "</b>"
+		else:
+			times = "<b>You've never been here</b>"
+
 		checkin_button = QPushButton("Check-in")
 		self.connect(checkin_button, SIGNAL("clicked()"), self.checkin)
 
-		name = "<b>" + venue[u'name'] + "</b>"
-		if len(venue[u'categories']) > 0:
-			name += " (" + venue[u'categories'][0][u'name'] + ")"
-
 		i = 0
-		gridLayout.addWidget(checkin_button, i, 0, 1, 3)
+		gridLayout.addWidget(checkin_button, i, 0, 1, 2)
 		i += 1
 		self.shout_text = QLineEdit(self)
 		self.shout_text.setPlaceholderText("Shout")
-		gridLayout.addWidget(self.shout_text, i, 0, 1, 3)
+		gridLayout.addWidget(self.shout_text, i, 0, 1, 2)
 		i += 1
-		gridLayout.addWidget(QLabel(name, self), i, 0)
+		gridLayout.addWidget(QLabel(name, self), i, 0, 1 ,2)
 		i += 1
-		gridLayout.addWidget(QLabel(address, self), i, 0)
+		gridLayout.addWidget(QLabel(address, self), i, 0, 1 ,2)
 		i += 1
-		gridLayout.addWidget(QLabel(address2, self), i, 0)
+		gridLayout.addWidget(QLabel(address2, self), i, 0, 1 ,2)
 		for item in venue[u'categories']:
-			if item[u'primary'] == "true":
+			if u'primary' in item and item[u'primary'] == "true":
 				i += 1
-				gridLayout.addWidget(QLabel(item[u'name'], self), i, 0)
+				gridLayout.addWidget(QLabel(item[u'name'], self), i, 0, 1, 2)
+		i += 1
+		line = QFrame()
+		line.setFrameShape(QFrame.HLine)
+		gridLayout.addWidget(line, i, 0, 1, 2)
+
+		if u'description' in venue:
+			i += 1
+			gridLayout.addWidget(QLabel(venue[u'description'], self), i, 0, 1, 2)
+			i += 1
+			line = QFrame()
+			line.setFrameShape(QFrame.HLine)
+			gridLayout.addWidget(line, i, 0, 1, 2)
+
+		i += 1
+		gridLayout.addWidget(QLabel(times, self), i, 0)
 		i += 1
 		gridLayout.addWidget(QLabel("Total Checkins: " + str(venue[u'stats'][u'checkinsCount']), self), i, 0)
-		i += 1
-		gridLayout.addWidget(QLabel("Total Users: " + str(venue[u'stats'][u'usersCount']), self), i, 0)
-		if u'beenHere' in venue:
-			if venue[u'beenHere'] == 1:
-				times = "once"
+		gridLayout.addWidget(QLabel("Total Visitors: " + str(venue[u'stats'][u'usersCount']), self), i, 1)
+
+		if u'hereNow' in venue:
+			hereNow = venue[u'hereNow'][u'count']
+			if hereNow == 0:
+				hereNow = "There's no one here now."
+			elif hereNow == 1:
+				hereNow = "There's just one person here now."
 			else:
-				times = str(venue[u'beenHere']) +" times"
+				hereNow = "There are " + repr(hereNow) + " people here now."
 			i += 1
-			gridLayout.addWidget(QLabel("You've been here " + times, self), i, 0)
-
-		full_venue_button = QPushButton("More... (TODO)")
+			gridLayout.addWidget(QLabel(hereNow, self), i, 0)
 
 		i += 1
-		gridLayout.addWidget(full_venue_button, i, 0, 1, 3)
+		if u'phone' in venue[u'contact']:
+			phoneCallButton = QPushButton("Call")
+			phoneCallButton.setIcon(QIcon.fromTheme("general_call"))
+			self.connect(phoneCallButton, SIGNAL("clicked()"), self.startPhoneCall)
+			gridLayout.addWidget(phoneCallButton, i, 0)
+				
+		if u'url' in venue:
+			websiteButton = QPushButton("Visit Website")
+			websiteButton.setIcon(QIcon.fromTheme("general_web"))
+			self.connect(websiteButton, SIGNAL("clicked()"), self.openUrl)
+			gridLayout.addWidget(websiteButton, i, 1)
+
+		if u'mayor' in venue:
+			mayorName = venue[u'mayor'][u'user'][u'firstName']
+			mayorCount = venue[u'mayor'][u'count']
+			mayorText = mayorName + " is the mayor with " + str(mayorCount) + " checkins!"
+			mayorButton = QPushButton()
+			mayorButton.setText(mayorText)
+			mayorButton.setIcon(QIcon(foursquare.image(venue[u'mayor'][u'user'][u'photo'])))
+			i += 1
+			gridLayout.addWidget(mayorButton, i, 0, 1, 2, Qt.AlignHCenter)
+
+		# TODO: menu
+		# TODO: specials
+
+		if u'tips' in venue:
+			i += 1
+			if venue[u'tips'][u'count'] == 0:
+				gridLayout.addWidget(QLabel("<b>There isn't a single tip!</b>", self), i, 0)
+			else:
+				if venue[u'tips'][u'count'] == 1:
+					gridLayout.addWidget(QLabel("<b>Just one tip</b>", self), i, 0)
+				else:
+					gridLayout.addWidget(QLabel("<b>" + str(venue[u'tips'][u'count']) + " tips</b>", self), i, 0)
+				for group in venue[u'tips'][u'groups']:
+					for tip in group[u'items']:
+						i += 1
+						gridLayout.addWidget(Tip(tip), i, 0, 1, 2)
+						i += 1
+						line = QFrame()
+						line.setFrameShape(QFrame.HLine)
+						line.setMaximumWidth(QApplication.desktop().screenGeometry().width() * 0.5)
+						gridLayout.addWidget(line, i, 0, 1, 2)
+
+			i += 1
+			gridLayout.addWidget(NewTipWidget(venue[u'id']), i, 0, 1 ,2)
+
+		if not fullDetails:
+			info_button_label = "More details (plus tips and stuff)"
+		else:
+			info_button_label = "Force refresh (updates cached data)"
+		more_info_button = QPushButton(info_button_label)
+		self.connect(more_info_button, SIGNAL("clicked()"), self.more_info)
+
+		i += 1
+		gridLayout.addWidget(more_info_button, i, 0, 1, 2)
+
+		hideWaitingDialog = Signal()
+		self.connect(self, SIGNAL("hideWaitingDialog()"), self.__hideWaitingDialog)
+
+		showWaitingDialog = Signal()
+		self.connect(self, SIGNAL("showWaitingDialog()"), self.__showWaitingDialog)
+
+		showMoreInfo = Signal()
+		self.connect(self, SIGNAL("showMoreInfo()"), self.more_info)
+
+		self.waitDialog = WaitingDialog(self)
+
+	def startPhoneCall(self):
+		QDesktopServices.openUrl("tel:" + self.venue[u'contact']['phone'])
+
+	def openUrl(self):
+		QDesktopServices.openUrl(self.venue[u'url'])
+
+	def __showWaitingDialog(self):
+		self.waitDialog.exec_()
+
+	def __hideWaitingDialog(self):
+		self.waitDialog.hide()
+
+	def more_info(self):
+		if not self.fullDetails:
+			venue = foursquare.venues_venue(self.venue['id'], Cache.CacheOrNull)
+			if venue:
+				self.close()
+				v = VenueDetailsWindow(self.parent(), venue, True).show()
+				return
+
+		# FIXME!
+		self.fullDetails = False
+		VenueDetailsThread(self.venue['id'], self).start()
+		self.__showWaitingDialog()
 
 	def checkin(self):
 		c = CheckinConfirmation(self, self.venue)
@@ -257,12 +419,34 @@ class VenueDetailsWindow(QMainWindow):
 		if c.buttonRole(c.clickedButton()) == QMessageBox.YesRole:
 			try:
 				ll = LocationProvider().get_ll(self.venue)
-				response = checkin(self.venue, ll, self.shout_text.text())
+				response = foursquare.checkin(self.venue, ll, self.shout_text.text())
 				CheckinDetails(self, response).show()
 			except IOError:
 	 			self.ibox = QMaemo5InformationBox()
 	 			self.ibox.information(self, "Network error; check-in failed. Please try again.", 15000)
 
+class VenueDetailsThread(QThread):	
+	def __init__(self, venueId, parent):
+		super(VenueDetailsThread, self).__init__(parent)
+		self.__parent = parent
+		self.venueId = venueId
+		
+	def run(self):
+		try:
+			venue = foursquare.venues_venue(self.venueId, Cache.ForceFetch)
+			if u'mayor' in venue:
+				print "there's a mayor!"
+				foursquare.image(venue[u'mayor'][u'user'][u'photo'])
+			self.__parent.hideWaitingDialog.emit()
+			# This tiny sleep in necesary to (a) Avoid an Xorg warning, (b) achieve a smoother transition
+			time.sleep(0.15)
+			self.__parent.showMoreInfo.emit()
+		except IOError:
+			self.__parent.hideWaitingDialog.emit()
+			self.__parent.networkError.emit()
+		self.exec_()
+		self.exit(0)
+
 class CategoryModel(QAbstractListModel):
 	def __init__(self, categories):
 		super(CategoryModel, self).__init__()
@@ -445,5 +629,5 @@ class NewVenueWindow(QMainWindow):
 			msgBox.setWindowTitle("Venue added")
 			msgBox.exec_()
 
-			v = VenueDetailsWindow(self, response[u'response'][u'venue'])
+			v = VenueDetailsWindow(self, response[u'response'][u'venue'], True)
 			v.show()
\ No newline at end of file
